<?php

if (!defined('ABSPATH')) {
    exit;
}

abstract class TransactionCloudType
{
    protected $properties = [];

    /**
     * Defines the array of available properties
     *
     * @return array
     */
    abstract public static function getAvailableProperties();

    /**
     * Defines the array of required properties
     *
     * @return array
     */
    abstract public static function getRequiredProperties();

    /**
     * Creates object from given API's JSON string
     *
     * @param string $json
     *
     * @return self
     */
    public static function make($json)
    {
        $properties = json_decode($json, true, 512, JSON_THROW_ON_ERROR);

        $object = new static();

        if (!static::hasRequiredProperties($properties)) {
            throw new TransactionCloudMapException('Could not map json string to object');
        }

        foreach ($properties as $key => $value) {
            $object->properties[$key] = $value;
        }

        return $object;
    }

    /**
     * Determines if object has all required properties
     *
     * @param array $data
     *
     * @return bool
     */
    public static function hasRequiredProperties($data)
    {
        foreach (static::getRequiredProperties() as $field) {
            if (!isset($data[$field])) {
                return false;
            }
        }

        return true;
    }

    public function __get($name)
    {
        if (!in_array($name, static::getAvailableProperties())) {
            throw new RuntimeException($name . ' can not be accessed');
        }

        if (isset($this->properties[$name])) {
            return $this->properties[$name];
        }

        return null;
    }
}
